{shared{
  open Eliom_lib
  open Eliom_content
}}

module Chat1_app =
  Eliom_registration.App (
    struct
      let application_name = "chat"
    end)

let main_service =
  Eliom_service.service ~path:[] ~get_params:Eliom_parameter.unit ()

let bus = Eliom_bus.create Json.t<string>

let user =
  Eliom_reference.eref ~scope:Eliom_common.default_session_scope None

let login_service = Eliom_registration.Action.register_post_coservice'
  ~post_params:Eliom_parameter.(string "username" ** string "password")
  (fun () (username, password) ->
    if username = password then
      Eliom_reference.set user (Some username)
    else
      Lwt.return ())

let logout_service = Eliom_registration.Action.register_post_coservice'
  ~post_params:Eliom_parameter.unit
  (fun () () ->
    Eliom_state.discard ~scope:Eliom_common.default_session_scope ())

let () =
  Chat1_app.register
    ~service:main_service
    (fun () () ->
      match_lwt Eliom_reference.get user with
        | None -> (* not logged in *)
          Lwt.return
            (Eliom_tools.F.html
               ~title:"login!"
               Html5.F.(body [
                 post_form ~service:login_service
                   (fun (username_nm, password_nm) -> Html5.F.([
                     string_input ~input_type:`Text ~name:username_nm ();
                     string_input ~input_type:`Password ~name:password_nm ();
                     input ~input_type:`Submit ~value:"Login" ();
                   ])) ()
               ]))
        | Some user ->
          let messages = Html5.D.(ul []) in
          let prompt = Html5.D.(input ~input_type:`Text ()) in
          ignore {unit{
            let prompt_dom = Html5.To_dom.of_input %prompt in
            let keys =
              Lwt_js_events.keypresses
                prompt_dom
                (fun ev _ ->
                  if ev##keyCode = 13 then
                    ( let str = prompt_dom##value in
                      prompt_dom##value <- Js.string "";
                      Eliom_bus.write %bus (Js.to_string str) )
                  else
                    Lwt.return ())
            in
            let msgs =
              Lwt_stream.iter
                (fun msg ->
                  Html5.Manip.appendChild
                    %messages
                    Html5.F.(li [pcdata msg]))
                (Eliom_bus.stream %bus)
            in
            Eliom_client.onunload
              (fun () ->
                Lwt.cancel keys;
                Lwt.cancel msgs)
          }};
          let logout =
            Html5.F.(post_form ~service:logout_service
                       (fun () -> [
                         input ~input_type:`Submit ~value:"Logout" ()
                       ]) ())
          in
          Lwt.return
            (Eliom_tools.F.html
               ~title:"chat"
               ~css:[["css";"chat.css"]]
               Html5.F.(body [
                 h2 [Printf.ksprintf pcdata "Welcome, %s!" user];
                 logout;
                 messages;
                 prompt;
               ])))
